#!/usr/bin/env python
#v0.1
#March 19 2007
#adam wolf
#adamwolf@feelslikeburning.com
#http://feelslikeburning.com

from optparse import OptionParser
from pysqlite2 import dbapi2 as sqlite
import os.path
import sys

class Action:
    def __init__(self):
	name = "uninitialized"
	
    def __repr__(self):
	return self.name

class Context:
    def __init__(self):
	name = "uninitalized"

    def __repr__(self):
	return self.name

class Project:
    def __init__(self):
	name = "uninitalized"

    def __repr__(self):
	return self.name

def parseActiontasticSqlite(filename, sortType):
    
    if sortType == "context":
        action_query = "select ZNAME, ZPROJECT, ZCONTEXT from ZACTION where ZCOMPLETED = 0 order by ZCONTEXT, ZCONTEXTORDER"
    elif sortType == "project":
        action_query = "select ZNAME, ZPROJECT, ZCONTEXT from ZACTION where ZCOMPLETED = 0 order by ZPROJECT, ZPROJECTORDER"
    else:
	    raise Exception
    
    context_query = "select Z_PK, ZNAME from ZCONTEXT"
    project_query = "select Z_PK, ZNAME from ZPROJECT"
    context_names = {}
    project_names = {}

    actions = []
    
    con = sqlite.connect(filename)
    cur = con.cursor()
    
    cur.execute(context_query)
    for (id, name) in cur:
	    context_names[id] = name
    
    cur.execute(project_query)
    for (id, name) in cur:
	    project_names[id] = name

    contexts = {}
    for contextID in context_names:
	    contexts[context_names[contextID]] = []
	    
    projects = {}
    for projectID in project_names:
	    projects[project_names[projectID]] = []
    
    cur.execute(action_query)
    for (name, projectID, contextID) in cur:
        action = Action()
        project = project_names[projectID]
        context = context_names[contextID]
        action.name = name
        action.project = project
        action.context = context
        if sortType == "context":
            contexts[context].append(action)
        elif sortType == "project":
            projects[project].append(action)

    if sortType == "context":
        return contexts
    elif sortType == "project":
        return projects

def main():
    usage = """usage: %prog [-i INFILE] [-o OUTFILE] [-v] [-q] [-c] [-p]
    
    %prog exports the Actiontastic database to plaintext.
    Not all information is exported (dates, notes, etc.)  
    
    DO NOT USE THIS AS A BACKUP SOLUTION.
    
    If -f is omitted, ~/Library/Application Support/Actiontastic/Actiontastic.sqlite is assumed.
    If -o is omitted, standard out is assumed.
    If neither sort by context nor sort by project is specified, sort by context is assumed.
    
    More information at http://feelslikeburning.com/projects/actiontastic-exporter/"""
    
    parser = OptionParser(usage)
    parser.add_option("-i", "--input-file", type="string", dest="inputFilename", help="read data from INFILE", default = os.path.expanduser("~/Library/Application Support/Actiontastic/Actiontastic.sqlite"))
    parser.add_option("-o", "--output-file", type="string", dest="output", help="write output to file OUTFILE", default = None)
    parser.add_option("-c", "--sort-by-context", action="store_true", dest="sortByContext", default = False)
    parser.add_option("-p", "--sort-by-project", action="store_true", dest="sortByProject", default = False)
    parser.add_option("-v", "--verbose", action="store_true", help="be verbose about what the program is doing", dest="verbose")
    parser.add_option("-m", "--more-output", action="store_true", help="export both the project name and context name", dest="moreOutput")
    (options, args) = parser.parse_args()

    if options.verbose == True:
        print "Parsing command line arguments . . ."

    if options.sortByContext == True and options.sortByProject == True:
	    sys.stderr.write("Both --sort-by-context and --sort-by-project specified on command line. Exiting . . .\n")
	    sys.exit(-1)
    
    if options.sortByContext == False and options.sortByProject == False:
	    options.sortByContext = True

    if not os.path.isfile(options.inputFilename):
        sys.stderr.write("Error opening input file %s. Exiting. . .\n" % options.inputFilename)
        sys.exit(-1)
    
    if options.verbose == True:
        print "Using %s as input file" % options.inputFilename
        if options.output == None:
            print "Using stdout for output"
        else:
            print "Using file %s for output" % options.output
        if options.sortByContext == True:
            print "Sorting by context"
        elif options.sortByProject == True:
            print "Sorting by project"
        if options.verbose == True:
            print "Verboseness on"
        if options.moreOutput == True:
            print "Extra verbosity in the export"
        
    if options.sortByContext == True:
        sortType = "context"
    elif options.sortByProject == True:
        sortType = "project"
	
    groups = {}
#   groups is a dictionary.  It has keys which are names of contexts or projects and the value is a list of actions
    
    if options.verbose == True:
        print "Parsing sqlite database . . ."
    
    groups = parseActiontasticSqlite(options.inputFilename, sortType)

    if options.verbose == True:
        if options.output == None:
            print "Printing plain-text export to standard out."
        else:
            print "Printing plain-text export to file %s." % options.output    

    if options.output != None:
        try:
            outputFile = open(options.output, 'w')
        except:
            sys.stderr.write("Error opening file %s for writing. Exiting . . . \n" % options.output)
            sys.exit(1)
    
    for groupName in sorted(groups):
        if options.output == None:
            print groupName
        else:
            print >>outputFile, groupName
        for action in groups[groupName]:
            if options.moreOutput:
                if sortType == "context":
                    if options.output == None:
                        print "\t%s\t[%s]" % (action.name, action.project)
                    else:
                        print >>outputFile, "\t%s\t[%s]" % (action.name, action.project)
                if sortType == "project":
                    if options.output == None:
                        print "\t%s\t[%s]" % (action.name, action.context)
                    else:
                        print >>outputFile, "\t%s\t[%s]" % (action.name, action.context)
            else:
                if options.output == None:
                   print "\t%s" % action.name
                else:
                   print >>outputFile, "\t%s" % action.name


    if options.verbose == True:
        print "Finished export."

    if options.output != None:
        if options.verbose == True:
            print "Closing file."
        outputFile.close()

if __name__ == "__main__":
    main()
